package api

import (
	"net/http"
	"database/sql"
	"encoding/json"
	"strconv"
	"log"
	// "gontrol/src/logger"

	_ "github.com/go-sql-driver/mysql"
)

type Agent struct {
	AgentID        int    `json:"agentId"`
	AgentName      string `json:"agentName"`
	AgentType      string `json:"agentType"`
	InitialContact string `json:"initialContact"`
	LastContact    string `json:"lastContact"`
	IPv4Address    string `json:"IPv4Address"`
	Status         string `json:"status"`
	AddPort        string `json:"addPort"`
	HostName       string `json:"HostName"`
}

// var db 	 *sql.DB

func DeleteAgent(db *sql.DB, w http.ResponseWriter, r *http.Request, agentID string) ([]Agent, error) {
	query := "DELETE FROM agents WHERE agentId = ?"
	_, err := db.Exec(query, agentID)
	if err != nil {
		http.Error(w, "Failed to delete agent", http.StatusInternalServerError)
		return nil, err
	}

	agents, err := GetAgents(db)
	if err != nil {
		http.Error(w, "Failed to fetch agents", http.StatusInternalServerError)
		return nil, err
	}

	return agents, err
}

func CreateAgent(db *sql.DB, w http.ResponseWriter, r * http.Request) (http.ResponseWriter, error){
	err := r.ParseForm()
	if err != nil {
		http.Error(w, "Invalid form data", http.StatusBadRequest)
	}

	agentName := r.FormValue("agentName")
	agentId := r.FormValue("agentId")
	agentType := r.FormValue("agentType")
	IPv4Address := r.FormValue("IPv4Address")
	addPort :=r.FormValue("addPort")
	hostname := r.FormValue("hostname")
	// initalContact := r.FormValue("initialContact")
	// lastContact := r.FormValue("lastContact")

	// query := "INSERT INTO agents (agentId, agentName, agentType, IPv4Address, addPort, initialContact, lastContact) VALUES (?, ?, ?, ?, ?, NOW(), NOW())"
	query := "INSERT INTO agents (agentId, agentName, agentType, IPv4Address, addPort, hostName, initialContact, lastContact) VALUES (?, ?, ?, ?, ?, ?, CURRENT_TIMESTAMP, CURRENT_TIMESTAMP)"
	_, err = db.Exec(query, agentId, agentName, agentType, IPv4Address, addPort, hostname)
	if err != nil {
		log.Printf("Database err is: %s", err)
		http.Error(w, "Failed to create agent", http.StatusInternalServerError)
		return nil, err
	}

	w.WriteHeader(http.StatusCreated)
	return w, nil
}

func UpdateAgent(db *sql.DB, w http.ResponseWriter, r *http.Request, agentId string) (http.ResponseWriter, error) {
	err := r.ParseForm()
	if err != nil {
		http.Error(w, "Invalid form data", http.StatusBadRequest)
		return nil, err
	}

	agentName := r.FormValue("agentName")
	lastContact := r.FormValue("lastContact")
	IPv4Address := r.FormValue("IPv4Address")

	query := "UPDATE agents SET agentName = ?, IPv4Address = ?, lastContact = ? where agentId = ?"
	_, err = db.Exec(query, agentName, IPv4Address, lastContact, agentId)
	if err != nil {
		http.Error(w, "Failed to update agent", http.StatusInternalServerError)
		return nil, err
	}

	return w, nil
}

func GetAgents(db *sql.DB) ([]Agent, error) {
	query := "SELECT agentId, agentName, agentType, IPv4Address, addPort, hostName, initialContact, lastContact FROM agents"
	rows, err := db.Query(query)
	if err != nil {
		return nil, err
	}
	defer rows.Close()

	var agents []Agent
	for rows.Next() {
		var agent Agent
		err := rows.Scan(&agent.AgentID, &agent.AgentName, &agent.AgentType, &agent.IPv4Address, &agent.AddPort, &agent.HostName, &agent.InitialContact, &agent.LastContact)
		if err != nil {
			return nil, err
		}
		agents = append(agents, agent)
	}
	return agents, rows.Err()
}

func GetAgent(db *sql.DB, w http.ResponseWriter, r *http.Request, agentId string) (Agent, error) {
	query := "Select agentId, agentName, agentType, IPv4Address, addPort, hostName, initialContact, lastContact from agents where agentId = ?"
	var agent Agent
	err := db.QueryRow(query, agentId).Scan(&agent.AgentID, &agent.AgentName, &agent.AgentType, &agent.IPv4Address, &agent.AddPort, &agent.HostName, &agent.InitialContact, &agent.LastContact)
	if err == sql.ErrNoRows {
		http.Error(w, "Agent not found", http.StatusNotFound)
		return Agent{} , err
	} else if err != nil {
		http.Error(w, "Failed to fetch agent", http.StatusInternalServerError)
		return Agent{}, err
	}

	return agent, nil

}

func GetAgentNames(db *sql.DB, w http.ResponseWriter, r *http.Request) {
	query := "SELECT agentName from agents"
	rows, err := db.Query(query)
	if err != nil {
		http.Error(w, "Failed to fetch agent names", http.StatusInternalServerError)
		return
	}
	defer rows.Close()

	var agentNames []string
	for rows.Next() {
		var name string
		if err := rows.Scan(&name); err != nil {
			http.Error(w, "Error reading agent names", http.StatusInternalServerError)
			return
		}
		agentNames = append(agentNames, name)
	}

	w.Header().Set("Content-Type", "application/json")
	json.NewEncoder(w).Encode(agentNames)
}


func GetAgentIds(db *sql.DB, w http.ResponseWriter, r *http.Request) {
	query := "SELECT agentId from agents"
	rows, err := db.Query(query)
	if err != nil {
		http.Error(w, "Failed to fetch agent names", http.StatusInternalServerError)
		return
	}
	defer rows.Close()

	var agentIds []string
	for rows.Next() {
		var id int
		if err := rows.Scan(&id); err != nil {
			http.Error(w, "Error reading agent names", http.StatusInternalServerError)
			return
		}
		agentIds = append(agentIds, strconv.Itoa(id))
	}

	// ocket.go.Header().Set("Content-Type", "application/json")
	json.NewEncoder(w).Encode(agentIds)
}
